import fs from 'fs';
import path from 'path';
import qs from 'qs';
import moment from 'moment';
import axios, { AxiosInstance } from 'axios';
import { Injectable } from '@nestjs/common';
import { Cron, CronExpression } from '@nestjs/schedule';
import { taConfig, taUrl } from '../../config/ta.config';
import { NotifyService } from './notify.service';
import { INotifyConfig, IRecordInfo, ITaConfig } from './interface';
import { __ROOT__ } from '../../config/config';

@Injectable()
export class TaService {
  private axiosInstance: AxiosInstance;
  constructor(private _notifyService: NotifyService) {
    this.axiosInstance = axios.create({
      baseURL: taUrl
    });
  }
  @Cron(CronExpression.EVERY_HOUR)
  async checkAll() {
    const date = moment().format('YYYY-MM-DD');
    const config = this.readConfig();

    Object.values(taConfig).forEach(c => {
      this.checkXianYu(c, date, 'get', config.getXianYu).then();
      this.checkXianYu(c, date, 'use', config.useXianYu).then();
      this.checkPay(c, date, config.recharge).then();
      config.getItem.map(([itemId, limit]) => this.checkItem(c, date, itemId, 'get', limit));
      config.useItem.map(([itemId, limit]) => this.checkItem(c, date, itemId, 'use', limit));
    });
  }

  private async checkPay(config: ITaConfig, date: string, limit: number) {
    if (!limit) {
      console.log('none checkPay config', limit);
      return;
    }

    const { eventDb, apiSecret, channel } = config;
    const sql = this.rechargeSql(eventDb, date, limit);
    const res = await this.readData(apiSecret, sql);
    await this._notifyService.notifyRecharge(res, channel);
  }

  private async checkXianYu(config: ITaConfig, date: string, type: 'use' | 'get', limit: number) {
    if (!limit) {
      console.log(`none checkGetXianYu config, limit: ${limit}`);
      return;
    }

    const { eventDb, apiSecret, channel } = config;
    const eventName = type === 'use' ? 'use_xian_yu' : 'get_xian_yu';
    const sql = this.xianYuChangeSql(eventDb, date, eventName, limit);
    const res = await this.readData(apiSecret, sql);
    await this._notifyService.notifyXianYu(res, channel, type);
  }

  private async checkItem(config: ITaConfig, date: string, itemId: number, type: 'use' | 'get', limit: number) {
    if (!limit) {
      console.log(`none checkItem config, type: ${type}, limit: ${limit}`);
      return;
    }

    const { eventDb, apiSecret, channel } = config;
    const eventName = type === 'use' ? 'use_item' : 'get_item';
    const sql = this.itemChangeSql(eventDb, date, itemId, eventName, limit);
    const res = await this.readData(apiSecret, sql);

    await this._notifyService.notifyItem(res, channel, itemId, type);
  }

  private readConfig() {
    const file = path.resolve(__ROOT__, 'temp/notify.txt');
    const str = fs.readFileSync(file).toString();
    return JSON.parse(str) as INotifyConfig;
  }

  private async readData(token: string, sql: string) {
    const { data } = await this.axiosInstance.post(`querySql?token=${token}`, qs.stringify({
      sql,
      format: 'csv_header'
    }), {
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded'
      }
    });
    // console.log('data', data);
    return this.formatData(data);
  }

  private formatData(str: string) {
    const res: IRecordInfo[] = [];
    const arr = str.replaceAll('"', '').split('\n');
    const header = arr.shift().split(',');

    for (const s of arr) {
      if (!s) continue;

      const ss = s.split(',');
      const temp: IRecordInfo = {} as IRecordInfo;
      for (const idx in ss) {
        temp[header[idx]] = isNaN(+ss[idx]) ? ss[idx] : +ss[idx];
      }

      res.push(temp);
    }

    return res;
  }

  private rechargeSql(db: string, date: string, limit = 1000) {
    return `
      SELECT "#account_id" AS "uid", sum ("pay_amount") AS "num"
      FROM ${db}
      WHERE "$part_date" = '${date}' AND "#event_name" = 'order_finish'
      GROUP BY "#account_id" HAVING sum ("pay_amount") > ${limit}
    `;
  }

  private xianYuChangeSql(db: string, date: string, eventName: string, limit = 10 * 1000) {
    return `
      SELECT "#account_id" AS "uid", sum ("change_num") AS "num"
      FROM ${db} 
      WHERE "$part_date" = '${date}' AND "#event_name" = '${eventName}'
      GROUP BY "#account_id" HAVING sum ("change_num") > ${limit}
    `;
  }

  private itemChangeSql(db: string, date: string, itemId: number, eventName: string, limit = 100000) {
    return `
      SELECT "#account_id" AS "uid", sum ("change_num") AS "num"
      FROM ${db} 
      WHERE "$part_date" = '${date}' AND "#event_name" = '${eventName}' AND "item_id" = ${itemId}
      GROUP BY "#account_id" HAVING sum ("change_num") > ${limit}
    `;
  }
}
